#!/usr/bin/python

# PlaneDomains Copyright (C) 2013  Marco Spadini
#
# PlaneDomains is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PlaneDomains is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with StartingPoints-1D.  If not, see <http://www.gnu.org/licenses/>.
# 
# 
# About the author:
# Marco Spadini                         
# Dipartimento di Matematica e Informatica,
# Via S. Marta 3, 50139 Firenze,       
# ITALY 
# 
# web: http:\\www.dma.unifi.it\~spadini
# e-mail: marco.spadini@unifi.it
#
#
#  Description:  
#  The purpose of this program is to sketch plane domains defined by a
#  complex sets of inequalities. 
#  The user should provide a reference rectangle R (determined by two opposite 
#  corners, south-west SW=(x0,y0) and north-east NE=(x1,y1)), and a set of 
#  conditions (namely a bunch of inequalities joined by logical connectives as 
#  ".and.", ".or.", ".xor.", or the operator ".not." ). The set of conditions can 
#  be distributed on several lines (all logically linked by ".and.", but do **not**
#  break a single condition over two lines: results are unpredictable).
#  The program tries to sketch the closure of the interior of the set D of those 
#  points (x,y) in the reference rectangle R where the condition holds. (Note that 
#  boundaries, albeit shown, are not necessarily included in the domain D, but do 
#  belong to the closure of D).
#
#  The mathematical notation allowes the Fortran 90/95 standard functions and intrinsics
#  (with the addition of ".xor." and "signum").
#


from Tkinter import *
from math import *
import sys, os, string, time, commands

version= '0.5'

root = Tk()
f_entry=[]
blabla = StringVar()
blabla.set('  ..... READY .....  ')
(stt, opF) = commands.getstatusoutput("gcc -dumpversion")
if stt ==0:
  print "found gcc version ", opF
(stt, output) = commands.getstatusoutput("gfortran -v")
if stt:
  print "gfortran not found: exiting"
  sys.exit(1)
if opF >= '4.6':
  opF = '-Ofast'
else:
  opF = '-O3'
print 'Optimization flag set to ', opF  
  

def incrementa():
  global f_entry,defaultTxt
  f_entry.append(Entry(vtop, width=62))
  f_entry[len(f_entry)-1].pack(side='top')
  f_entry[len(f_entry)-1].focus_set()
  root.update()
  
def decrementa():
  global f_entry
  if len(f_entry)>1:
    f_entry[len(f_entry)-1].pack_forget()
    f_entry.pop()
    root.update()
  else:
    message('Illegal decrement')
    time.sleep(2)
    message('  ...... READY ......  ')
  f_entry[len(f_entry)-1].focus_set()
    
  
root.title('PlaneDomains %s'% version)

toptop = Frame(root); toptop.pack(padx=10,pady=10)
Label(toptop, text="""Draws within the given reference rectangle of opposite corners,  SW=(x0,y0) and\n\
NE=(x1,y1), the closure of the interior of the set of the points (x,y) where all given\n\
conditions hold.""" ).pack(side='top')


top = Frame(root); top.pack()

vtop = LabelFrame(top, text='Conditions')
vtopt = Frame(vtop,width=100)
vtopt.pack(side='top',expand='true')
cond_add = Button(vtopt, text = 'ADD line', fg='blue', command=incrementa)
cond_add.pack(side=LEFT,pady='5',padx=3)
cond_sub = Button(vtopt, text = 'REMOVE line', fg='blue', command=decrementa)
cond_sub.pack(side=RIGHT,pady='5',padx=3)
vtop.pack()
f_entry.append(Entry(vtop, width=62))
f_entry[0].pack(side='top')
f_entry[0].focus_set()
        
rtop=LabelFrame(top, text='Reference rectangle')
rtop.pack(ipadx=5,ipady=5)

Label(rtop, text='  SW:').pack(side='left')
Label(rtop, text='  x0 =').pack(side='left')
x0_entry = Entry(rtop, width=4)
x0_entry.pack(side='left')
x0_entry.insert('end', '-3.') #default value

Label(rtop, text='  y0 =').pack(side='left')
y0_entry = Entry(rtop, width=4)
y0_entry.pack(side='left')
y0_entry.insert('end', '-3.') #default value

Label(rtop, text='         NE:').pack(side='left')
Label(rtop, text='  x1 =').pack(side='left')
x1_entry = Entry(rtop, width=4)
x1_entry.pack(side='left')
x1_entry.insert('end', '3.') #default value

Label(rtop, text='  y1 =').pack(side='left')
y1_entry = Entry(rtop, width=4)
y1_entry.pack(side='left')
y1_entry.insert('end', '3.') #default value

mid = Frame(root); mid.pack()
Label(mid, text=' % resolution = ').pack(side='left')
n_entry = Scale(mid, orient='horizontal', from_= 1, to=100, tickinterval=99, resolution=1, length= 320)
n_entry.pack(side='left',padx=5)
n_entry.set(20) #default value

bottom = LabelFrame(root, text='System messages')
bottom.pack(ipadx='25')
Label(bottom, textvariable = blabla, fg='red').pack(padx='25')

def message(string):
  blabla.set(string)
  root.update_idletasks()

def calc(event=None):
  message('  .......  BUSY  .......  ')
  funzioni = filter(lambda x: x.get().strip() != '',f_entry[::-1])
  try:
    funz = funzioni.pop().get()
  except Exception, nam:
    message('Error getting condition')
    print nam
    return
  funz = '('+funz  
  for i in funzioni:
    funz += ') .and. (' + i.get()
  funz += ')'  
  progfun=funz.replace(".xor.",".neqv.")  
  progfun=funz.replace("signum(","sign(1.,")  
  try:  
    x0 = str(eval(x0_entry.get()))
    x1 = str(eval(x1_entry.get()))
    y0 = str(eval(y0_entry.get()))
    y1 = str(eval(y1_entry.get()))
  except Exception, nam:
    message(' Some data do not make sense ')
    print nam
    return
  n = str(n_entry.get()*500)
  try:
    os.remove('disegno.dat') 
  except:
    pass
  try:
    os.remove('funzione.f90') 
  except:
    pass
  f=open('funzione.f90','w')
  f.write(""" function f(xi)
       real, dimension(2) :: xi
       logical :: f
       real :: x,y
       x=xi(1)
       y=xi(2)

       """ +"  f="+progfun+"\n end function f\n")
  f.close()
  failure=os.system('gfortran %s -march=native -mtune=native dom2.f03 funzione.f90'% opF)
  if failure:
    print 'Compilation failed\n'
    message('Syntax error/ badly formed condition')
    return
  failure=os.system('./a.out '+x0+" "+y0+" "+x1+" "+y1+" "+n)
  if failure:
    print 'Execution failure'
    message('Execution failure')
    return
  f = open('disegno.gnuplot', 'w')
  f.write("unset label; set autoscale; ")
  f.write("set title '"+funz+"'; ")
  f.write("set xrange ["+x0+":"+x1+"] ; ")
  f.write("set yrange ["+y0+":"+y1+"] ; ")
  f.write("plot 'disegno.dat' using 1:2 notitle with dots ;")
  f.close()
  failure=os.system('gnuplot -persist disegno.gnuplot')
  if failure:
    print 'Plotting failure'
    message('Plotting failure')
    return
  message('  .....  READY  ......  ')
    
 
def smetti(event=None):
  try: 
    os.remove('disegno.dat') 
  except:
    pass
  try: 
    os.remove('funzione.f90') 
  except:
    pass
  try: 
    os.remove('a.out') 
  except:
    pass
  try: 
    os.remove('disegno.gnuplot') 
  except:
    pass
  print " 'Bye"
  root.quit() 

Button(mid, text=' COMPUTE ', relief='raised', fg='blue',command=calc).pack(side='left')
Button(mid, text=' QUIT ', relief='raised', fg='red', command=smetti).pack(side='left')

#root.bind('<q>', quit)   
root.bind('<Return>', calc)
root.mainloop()
